from typing import List

from bluer_options.terminal import show_usage, xtra


def help_(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("yes | no | empty | non-empty", mono=mono)

    return show_usage(
        [
            "@assert",
            "<this>",
            "<that>",
            f"[{options}]",
        ],
        "assert <this> <?> <that>.",
        mono=mono,
    )


def help_list(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@assert",
            "list",
            "<this,that,which>",
            "<that,this,what>",
        ],
        "assert that the two lists are identical.",
        mono=mono,
    )


help_functions = {
    "": help_,
    "list": help_list,
}



from typing import List

from bluer_options.terminal import show_usage


def help_diagnose(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@wifi",
            "diagnose",
        ],
        "diagnose wifi.",
        mono=mono,
    )


def help_get_ssid(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@wifi",
            "get_ssid",
        ],
        "get wifi ssid.",
        mono=mono,
    )


help_functions = {
    "diagnose": help_diagnose,
    "get_ssid": help_get_ssid,
}



from bluer_ai.help.generic import help_functions as generic_help_functions
from bluer_ai.help.actions import help_perform_action
from bluer_ai.help.assert_ import help_functions as help_assert
from bluer_ai.help.blueness import help_blueness
from bluer_ai.help.browse import help_browse
from bluer_ai.help.code import help_code
from bluer_ai.help.conda import help_functions as help_conda
from bluer_ai.help.env import help_functions as help_env
from bluer_ai.help.eval import help_eval
from bluer_ai.help.git import help_functions as help_git
from bluer_ai.help.gpu import help_functions as help_gpu
from bluer_ai.help.help import help_help
from bluer_ai.help.hr import help_hr
from bluer_ai.help.init import help_init
from bluer_ai.help.instance import help_functions as help_instance
from bluer_ai.help.latex import help_functions as help_latex
from bluer_ai.help.logging import help_cat
from bluer_ai.help.logging import help_functions as help_log
from bluer_ai.help.list import help_functions as help_list
from bluer_ai.help.not_ import help_not
from bluer_ai.help.open import help_open
from bluer_ai.help.options import help_functions as help_option
from bluer_ai.help.pause import help_pause
from bluer_ai.help.plugins import help_functions as help_plugins
from bluer_ai.help.repeat import help_repeat
from bluer_ai.help.sagemaker import help_functions as help_sagemaker
from bluer_ai.help.screen import help_functions as help_screen
from bluer_ai.help.seed import help_functions as help_seed
from bluer_ai.help.select import help_select
from bluer_ai.help.session import help_functions as help_session
from bluer_ai.help.ssh import help_functions as help_ssh
from bluer_ai.help.string import help_functions as help_string
from bluer_ai.help.storage import help_functions as help_storage
from bluer_ai.help.sleep import help_sleep
from bluer_ai.help.source import (
    help_source_caller_suffix_path,
    help_source_path,
)
from bluer_ai.help.terminal import help_badge
from bluer_ai.help.terraform import help_functions as help_terraform
from bluer_ai.help.wait import help_wait
from bluer_ai.help.watch import help_watch
from bluer_ai.help.wifi import help_functions as help_wifi

help_functions = generic_help_functions(plugin_name="bluer_ai")


help_functions.update(
    {
        "assert": help_assert,
        "badge": help_badge,
        "blueness": help_blueness,
        "browse": help_browse,
        "cat": help_cat,
        "code": help_code,
        "conda": help_conda,
        "env": help_env,
        "eval": help_eval,
        "git": help_git,
        "gpu": help_gpu,
        "help": help_help,
        "hr": help_hr,
        "init": help_init,
        "instance": help_instance,
        "latex": help_latex,
        "log": help_log,
        "list": help_list,
        "not": help_not,
        "open": help_open,
        "option": help_option,
        "pause": help_pause,
        "perform_action": help_perform_action,
        "plugins": help_plugins,
        "repeat": help_repeat,
        "sagemaker": help_sagemaker,
        "screen": help_screen,
        "seed": help_seed,
        "select": help_select,
        "sleep": help_sleep,
        "session": help_session,
        "storage": help_storage,
        "source_caller_suffix_path": help_source_caller_suffix_path,
        "source_path": help_source_path,
        "ssh": help_ssh,
        "string": help_string,
        "terraform": help_terraform,
        "wait": help_wait,
        "watch": help_watch,
        "wifi": help_wifi,
    }
)



from bluer_options.env import load_config, load_env, get_env

load_env(__name__)
load_config(__name__)


abcli_is_github_workflow = get_env("GITHUB_ACTIONS")

abcli_display_fullscreen = get_env("abcli_display_fullscreen")

BLUER_AI_GIT_SSH_KEY_NAME = get_env("BLUER_AI_GIT_SSH_KEY_NAME")

bluer_ai_gpu_status_cache = get_env("bluer_ai_gpu_status_cache")

abcli_path_abcli = get_env("abcli_path_abcli")

ABCLI_PATH_IGNORE = get_env("ABCLI_PATH_IGNORE")

ABCLI_MLFLOW_STAGES = get_env("ABCLI_MLFLOW_STAGES")

BLUER_AI_GITHUB_TOKEN = get_env("BLUER_AI_GITHUB_TOKEN")



from typing import List

from bluer_options.terminal import show_usage


def help_(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@screen",
            "[<screen-name>]",
        ],
        "start a screen.",
        mono=mono,
    )


def help_detach(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@screen",
            "detach",
            "<screen-name>",
        ],
        "detach <screen-name>.",
        mono=mono,
    )


def help_list(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@screen",
            "list",
        ],
        "list screens.",
        mono=mono,
    )


def help_resume(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@screen",
            "resume",
            "<screen-name>",
        ],
        "resume <screen-name>",
        mono=mono,
    )


help_functions = {
    "": help_,
    "detach": help_detach,
    "list": help_list,
    "resume": help_resume,
}



from typing import List

from bluer_options.terminal import show_usage


def help_(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@option",
            '"$options"',
            "<keyword>",
            "[<default>]",
        ],
        "get $options[<keyword>].",
        mono=mono,
    )


def help_choice(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@option::choice",
            '"$options"',
            "<keyword-1,keyword-2,keyword-3>",
            "[<default>]",
        ],
        "return <keyword-*> that is in $options.",
        mono=mono,
    )


def help_int(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@option::int",
            '"$options"',
            "<keyword>",
            "[<default>]",
        ],
        "get int($options[<keyword>]).",
        mono=mono,
    )


help_functions = {
    "": help_,
    "choice": help_choice,
    "int": help_int,
}



from typing import List

from bluer_options.terminal import show_usage, xtra


def help_init(
    tokens: List[str],
    mono: bool,
) -> str:
    what = xtra("<plugin-name> | all | clear", mono=mono)

    options = xtra("clear,~terraform", mono=mono)

    return show_usage(
        [
            "@init",
            f"[{what}]",
            f"[{options}]",
        ],
        "init.",
        mono=mono,
    )



from typing import List

from bluer_options.terminal import show_usage, xtra


def help_clear(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "".join(
        [
            "cloud",
            xtra(",~dryrun", mono=mono),
        ]
    )

    return show_usage(
        [
            "@storage",
            "clear",
            f"[{options}]",
        ],
        "clear storage.",
        mono=mono,
    )


def help_status(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "count=<10>,depth=<2>"

    return show_usage(
        [
            "@storage",
            "status",
            f"[{options}]",
        ],
        "show storage status.",
        mono=mono,
    )


help_functions = {
    "clear": help_clear,
    "status": help_status,
}



from typing import List

from bluer_options.terminal import show_usage, xtra


def help_cat(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@cat",
            "<filename>",
        ],
        "cat <filename>.",
        mono=mono,
    )


def help_log(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@log",
            "<message>",
        ],
        "log message.",
        mono=mono,
    )


def help_log_error(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@error",
            "<message>",
        ],
        "error <message>.",
        mono=mono,
    )


def help_log_list(
    tokens: List[str],
    mono: bool,
) -> str:
    args = [
        '[--before "list of"]',
        '[--after "items(s)"]',
        "[--delim space | <delim>]",
    ]

    return show_usage(
        [
            "@log::list",
            "<this,that>",
        ]
        + args,
        "log list.",
        mono=mono,
    )


def help_log_rm(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@log",
            "rm",
        ],
        "rm the log.",
        mono=mono,
    )


def help_log_verbose(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "on | off"

    return show_usage(
        [
            "@log",
            "verbose",
            f"[{options}]",
        ],
        "verbose logging on/off.",
        mono=mono,
    )


def help_log_watch(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("rpi", mono=mono)

    return show_usage(
        [
            "@log",
            "watch",
            f"[{options}]",
            "[<machine-name>]",
        ],
        "watch the log.",
        mono=mono,
    )


def help_log_warning(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@warn",
            "<message>",
        ],
        "warn <message>.",
        mono=mono,
    )


help_functions = {
    "": help_log,
    "error": help_log_error,
    "warning": help_log_warning,
    "list": help_log_list,
    "rm": help_log_rm,
    "verbose": help_log_verbose,
    "watch": help_log_watch,
}



from typing import List

from bluer_options.terminal import show_usage


def help_add(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@ssh",
            "add",
            "<filename>",
        ],
        "ssh add <filename>.",
        mono=mono,
    )


def help_copy_id(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@ssh",
            "copy_id",
            "<filename>",
            "jetson_nano | rpi",
            "<machine-name>",
        ],
        "ssh copy-id <filename> to <machine-name>.",
        mono=mono,
    )


def help_ec2(
    tokens: List[str],
    mono: bool,
) -> str:
    options = "region=<region_1>,user=<ec2-user|ubuntu>,vnc"

    return show_usage(
        [
            "@ssh",
            "ec2",
            "<ip-address>",
            f"[{options}]",
        ],
        "ssh to <ip-address>.",
        mono=mono,
    )


def help_sbc(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@ssh",
            "jetson_nano | rpi",
            "<machine-name>",
        ],
        "ssh to <machine-name>.",
        mono=mono,
    )


def help_keygen(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@ssh",
            "keygen",
            "[<filename>]",
        ],
        "keygen <filename>.",
        mono=mono,
    )


help_functions = {
    "add": help_add,
    "copy_id": help_copy_id,
    "ec2": help_ec2,
    "sbc": help_sbc,
    "keygen": help_keygen,
}



from typing import List
import glob
import os

# https://chatgpt.com/c/683d39ac-34a8-8005-b780-71a6d2253ea9
try:
    from importlib.metadata import distributions
except ImportError:
    # for Python < 3.8
    from importlib_metadata import distributions  # type: ignore

from bluer_objects import file, path
from bluer_objects.env import abcli_path_git


def get_plugin_name(repo_name: str) -> str:
    return "abcli" if repo_name == "awesome-bash-cli" else repo_name.replace("-", "_")


def get_module_name(repo_name: str) -> str:
    list_of_candidates = sorted(
        file.path(filename)
        for filename in glob.glob(
            os.path.join(abcli_path_git, repo_name, "**/__init__.py"),
            recursive=True,
        )
    )

    if not list_of_candidates:
        return "no-module-found"

    return path.name(list_of_candidates[0])


def list_of_external(repo_names=False) -> List[str]:
    output = sorted(
        [
            repo_name
            for repo_name in [
                path.name(path_)
                for path_ in glob.glob(os.path.join(abcli_path_git, "*/"))
            ]
            if repo_name not in ["awesome-bash-cli", "bluer-ai"]
            and path.exists(
                os.path.join(
                    abcli_path_git,
                    repo_name,
                    get_module_name(repo_name),
                    ".abcli",
                )
            )
        ]
    )

    if not repo_names:
        output = [repo_name.replace("-", "_") for repo_name in output]

    return output


def list_of_installed(return_path: bool = False) -> List[str]:
    output = []

    for dist in distributions():
        try:
            name = dist.metadata.get("Name")
            if not name:
                continue

            key = name.lower().replace("-", "_")
            if key in ["abcli", "bluer_ai"]:
                continue

            # Estimate install path
            root_path = str(dist.locate_file(""))
            if "git" in root_path.split(os.sep):
                continue

            module_bash_folder = os.path.join(root_path, key, ".abcli")
            if not os.path.exists(module_bash_folder):
                continue

            output.append(module_bash_folder if return_path else key)

        except Exception:
            continue

    return output



from typing import List

from bluer_options.terminal import show_usage


def help_code(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@code",
            "<filename>",
        ],
        "code <filename>.",
        mono=mono,
    )



from typing import List

from bluer_options.terminal import show_usage


def help_after(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "bluer_ai_string_after",
            "<string>",
            "<substring>",
        ],
        "<string>.after(<substring>)",
        mono=mono,
    )


def help_before(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "bluer_ai_string_before",
            "<string>",
            "<substring>",
        ],
        "<string>.before(<substring>)",
        mono=mono,
    )


def help_random(
    tokens: List[str],
    mono: bool,
) -> str:
    args = [
        "[--float 1]",
        "[--int 1]",
        "[--length <8>]",
        "[--min <1.0>]",
        "[--max <100.0>]",
    ]

    return show_usage(
        [
            "@random",
        ]
        + args,
        "random float/int/string.",
        mono=mono,
    )


def help_timestamp(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@timestamp",
        ],
        "timestamp.",
        mono=mono,
    )


def help_timestamp_short(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@@timestamp",
        ],
        "short timestamp.",
        mono=mono,
    )


def help_today(
    tokens: List[str],
    mono: bool,
) -> str:
    args = [
        "[--include_time 1]",
        "[--unique 1]",
    ]

    return show_usage(
        [
            "@today",
        ]
        + args,
        "today.",
        mono=mono,
    )


help_functions = {
    "after": help_after,
    "before": help_before,
    "random": help_random,
    "timestamp": {
        "": help_timestamp,
        "short": help_timestamp_short,
    },
    "today": help_today,
}



import os

from bluer_options.help.functions import get_help
from bluer_objects import file, README

from bluer_ai import NAME, VERSION, ICON, REPO_NAME
from bluer_ai.help.functions import help_functions


def build():
    return all(
        README.build(
            path=os.path.join(file.path(__file__), path),
            ICON=ICON,
            NAME=NAME,
            VERSION=VERSION,
            REPO_NAME=REPO_NAME,
            MODULE_NAME=NAME,
            help_function=lambda tokens: get_help(
                tokens,
                help_functions,
                mono=True,
            ),
        )
        for path in [
            "..",
            # @ai
            "docs/aliases/conda.md",
            "docs/aliases/git.md",
            "docs/aliases/gpu.md",
            "docs/aliases/init.md",
            "docs/aliases/latex.md",
            "docs/aliases/logging.md",
            "docs/aliases/plugins.md",
            "docs/aliases/pypi.md",
            "docs/aliases/random.md",
            "docs/aliases/screen.md",
            "docs/aliases/seed.md",
            "docs/aliases/ssh.md",
            "docs/aliases/terraform.md",
            "docs/aliases/today.md",
            "docs/aliases/wifi.md",
            # @options
            "docs/aliases/assert.md",
            "docs/aliases/badge.md",
            "docs/aliases/browse.md",
            "docs/aliases/cat.md",
            "docs/aliases/code.md",
            "docs/aliases/env.md",
            "docs/aliases/eval.md",
            "docs/aliases/help.md",
            "docs/aliases/hr.md",
            "docs/aliases/list.md",
            "docs/aliases/not.md",
            "docs/aliases/open.md",
            "docs/aliases/option.md",
            "docs/aliases/pause.md",
            "docs/aliases/pylint.md",
            "docs/aliases/pytest.md",
            "docs/aliases/repeat.md",
            "docs/aliases/sleep.md",
            "docs/aliases/test.md",
            "docs/aliases/timestamp.md",
            "docs/aliases/wait.md",
            "docs/aliases/watch.md",
            # @objects
            "docs/aliases/select.md",
            "docs/aliases/storage.md",
        ]
    )
